"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaNtt = exports.SolanaNttWormholeTransceiver = exports.solanaNttProtocolFactory = void 0;
const anchor_1 = require("@coral-xyz/anchor");
const splToken = __importStar(require("@solana/spl-token"));
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
const sdk_solana_1 = require("@wormhole-foundation/sdk-solana");
const sdk_solana_core_1 = require("@wormhole-foundation/sdk-solana-core");
const bn_js_1 = __importDefault(require("bn.js"));
const index_js_1 = require("./anchor-idl/index.js");
const utils_js_1 = require("./utils.js");
function solanaNttProtocolFactory(token) {
    class _SolanaNtt extends SolanaNtt {
        tokenAddress = token;
        static async fromRpc(provider, config) {
            const [network, chain] = await sdk_solana_1.SolanaPlatform.chainFromRpc(provider);
            const conf = config[chain];
            if (conf.network !== network)
                throw new Error(`Network mismatch: ${conf.network} != ${network}`);
            if (!conf.tokenMap)
                throw new Error('Token map not found');
            const maybeToken = sdk_connect_1.tokens.filters.byAddress(conf.tokenMap, token);
            if (maybeToken === undefined)
                throw new Error('Token not found');
            if (!maybeToken.ntt)
                throw new Error('Token not configured with NTT');
            const { manager, transceiver } = maybeToken.ntt;
            return new _SolanaNtt(network, chain, provider, conf.contracts.coreBridge, {
                manager,
                transceiver: { wormhole: transceiver },
            });
        }
    }
    return _SolanaNtt;
}
exports.solanaNttProtocolFactory = solanaNttProtocolFactory;
class SolanaNttWormholeTransceiver {
    manager;
    address;
    constructor(manager, address) {
        this.manager = manager;
        this.address = address;
        //
    }
    async *receive(attestation, sender) {
        throw new Error('Method not implemented.');
    }
}
exports.SolanaNttWormholeTransceiver = SolanaNttWormholeTransceiver;
class SolanaNtt {
    network;
    chain;
    connection;
    wormholeId;
    contracts;
    core;
    xcvrs;
    program;
    pdas;
    config;
    constructor(network, chain, connection, wormholeId, contracts) {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.wormholeId = wormholeId;
        this.contracts = contracts;
        this.program = new anchor_1.Program(
        // @ts-ignore
        index_js_1.idl.ntt, this.contracts.manager, { connection });
        this.core = new sdk_solana_core_1.SolanaWormholeCore(network, chain, connection, {
            coreBridge: wormholeId,
        });
        this.pdas = (0, utils_js_1.nttAddresses)(this.program.programId);
        this.xcvrs = [new SolanaNttWormholeTransceiver(this, '')];
    }
    async getConfig() {
        this.config =
            this.config ??
                (await this.program.account.config.fetch(this.pdas.configAccount()));
        return this.config;
    }
    async *transfer(sender, amount, destination, queue) {
        const config = await this.getConfig();
        const outboxItem = web3_js_1.Keypair.generate();
        const senderAddress = new sdk_solana_1.SolanaAddress(sender).unwrap();
        const fromAuthority = senderAddress;
        const from = (0, spl_token_1.getAssociatedTokenAddressSync)(config.mint, fromAuthority);
        const transferArgs = {
            amount: new bn_js_1.default(amount.toString()),
            recipientChain: { id: (0, sdk_connect_1.toChainId)(destination.chain) },
            recipientAddress: Array.from(destination.address.toUniversalAddress().toUint8Array()),
            shouldQueue: queue,
        };
        const txArgs = {
            transferArgs,
            payer: senderAddress,
            from,
            fromAuthority,
            outboxItem: outboxItem.publicKey,
            config,
        };
        const transferIx = await (config.mode.locking !=
            null
            ? this.createTransferLockInstruction(txArgs)
            : this.createTransferBurnInstruction(txArgs));
        const releaseIx = await this.createReleaseOutboundInstruction({
            payer: senderAddress,
            outboxItem: outboxItem.publicKey,
            revertOnDelay: !queue,
        });
        const approveIx = splToken.createApproveInstruction(from, this.pdas.sessionAuthority(fromAuthority, transferArgs), fromAuthority, amount);
        const tx = new web3_js_1.Transaction();
        tx.feePayer = senderAddress;
        tx.add(approveIx, transferIx, releaseIx);
        yield this.createUnsignedTx({ transaction: tx, signers: [outboxItem] }, 'Ntt.Transfer');
    }
    async *redeem(attestations, payer) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        if (attestations.length !== this.xcvrs.length)
            throw 'No';
        // TODO: not this, we should iterate over the set of enabled xcvrs?
        const wormholeNTT = attestations[0];
        // Post the VAA that we intend to redeem
        yield* this.core.postVaa(payer, wormholeNTT);
        const senderAddress = new sdk_solana_1.SolanaAddress(payer).unwrap();
        const nttMessage = wormholeNTT.payload.nttManagerPayload;
        const emitterChain = wormholeNTT.emitterChain;
        const releaseArgs = {
            payer: senderAddress,
            config,
            nttMessage,
            recipient: new web3_js_1.PublicKey(nttMessage.payload.recipientAddress.toUint8Array()),
            chain: emitterChain,
            revertOnDelay: false,
        };
        const tx = new web3_js_1.Transaction();
        tx.feePayer = senderAddress;
        tx.add(await this.createReceiveWormholeMessageInstruction(senderAddress, wormholeNTT));
        tx.add(await this.createRedeemInstruction(senderAddress, wormholeNTT));
        tx.add(await (config.mode.locking != null
            ? this.createReleaseInboundUnlockInstruction(releaseArgs)
            : this.createReleaseInboundMintInstruction(releaseArgs)));
        yield this.createUnsignedTx({ transaction: tx }, 'Ntt.Redeem');
    }
    async getCurrentOutboundCapacity() {
        const rl = await this.program.account.outboxRateLimit.fetch(this.pdas.outboxRateLimitAccount());
        return BigInt(rl.rateLimit.capacityAtLastTx.toString());
    }
    async getCurrentInboundCapacity(fromChain) {
        const rl = await this.program.account.inboxRateLimit.fetch(this.pdas.inboxRateLimitAccount(fromChain));
        return BigInt(rl.rateLimit.capacityAtLastTx.toString());
    }
    async *completeInboundQueuedTransfer(fromChain, transceiverMessage, token, payer) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const senderAddress = new sdk_solana_1.SolanaAddress(payer).unwrap();
        const tx = new web3_js_1.Transaction();
        tx.feePayer = senderAddress;
        const releaseArgs = {
            payer: senderAddress,
            config,
            nttMessage: transceiverMessage,
            recipient: new web3_js_1.PublicKey(transceiverMessage.payload.recipientAddress.toUint8Array()),
            chain: fromChain,
            revertOnDelay: false,
        };
        tx.add(await (config.mode.locking != null
            ? this.createReleaseInboundUnlockInstruction(releaseArgs)
            : this.createReleaseInboundMintInstruction(releaseArgs)));
        yield this.createUnsignedTx({ transaction: tx }, 'Ntt.CompleteInboundTransfer');
    }
    async getInboundQueuedTransfer(chain, nttMessage) {
        const inboxItem = await this.program.account.inboxItem.fetch(this.pdas.inboxItemAccount(chain, nttMessage));
        if (!inboxItem)
            return null;
        const { recipientAddress, amount, releaseStatus } = inboxItem;
        const rateLimitExpiry = releaseStatus.releaseAfter
            ? releaseStatus.releaseAfter[0].toNumber()
            : 0;
        const xfer = {
            recipient: new sdk_solana_1.SolanaAddress(recipientAddress),
            amount: BigInt(amount.toString()),
            rateLimitExpiryTimestamp: rateLimitExpiry,
        };
        return xfer;
    }
    async createTransferLockInstruction(args) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const sessionAuthority = this.pdas.sessionAuthority(args.fromAuthority, args.transferArgs);
        const recipientChain = (0, sdk_connect_1.toChain)(args.transferArgs.recipientChain.id);
        return await this.program.methods
            .transferLock(args.transferArgs)
            .accounts({
            common: {
                payer: args.payer,
                config: { config: this.pdas.configAccount() },
                mint: config.mint,
                from: args.from,
                tokenProgram: config.tokenProgram,
                outboxItem: args.outboxItem,
                outboxRateLimit: this.pdas.outboxRateLimitAccount(),
            },
            peer: this.pdas.peerAccount(recipientChain),
            inboxRateLimit: this.pdas.inboxRateLimitAccount(recipientChain),
            custody: config.custody,
            sessionAuthority: sessionAuthority,
        })
            .instruction();
    }
    async createTransferBurnInstruction(args) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const recipientChain = (0, sdk_connect_1.toChain)(args.transferArgs.recipientChain.id);
        return await this.program.methods
            .transferBurn(args.transferArgs)
            .accounts({
            common: {
                payer: args.payer,
                config: { config: this.pdas.configAccount() },
                mint: config.mint,
                from: args.from,
                outboxItem: args.outboxItem,
                outboxRateLimit: this.pdas.outboxRateLimitAccount(),
            },
            peer: this.pdas.peerAccount(recipientChain),
            inboxRateLimit: this.pdas.inboxRateLimitAccount(recipientChain),
            sessionAuthority: this.pdas.sessionAuthority(args.fromAuthority, args.transferArgs),
        })
            .instruction();
    }
    async createReleaseOutboundInstruction(args) {
        const whAccs = sdk_solana_core_1.utils.getWormholeDerivedAccounts(this.program.programId, this.wormholeId);
        return await this.program.methods
            .releaseWormholeOutbound({
            revertOnDelay: args.revertOnDelay,
        })
            .accounts({
            payer: args.payer,
            config: { config: this.pdas.configAccount() },
            outboxItem: args.outboxItem,
            wormholeMessage: this.pdas.wormholeMessageAccount(args.outboxItem),
            emitter: whAccs.wormholeEmitter,
            transceiver: this.pdas.registeredTransceiver(this.program.programId),
            wormhole: {
                bridge: whAccs.wormholeBridge,
                feeCollector: whAccs.wormholeFeeCollector,
                sequence: whAccs.wormholeSequence,
                program: this.wormholeId,
            },
        })
            .instruction();
    }
    async createReceiveWormholeMessageInstruction(payer, wormholeNTT) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const nttMessage = wormholeNTT.payload.nttManagerPayload;
        const emitterChain = wormholeNTT.emitterChain;
        return await this.program.methods
            .receiveWormholeMessage()
            .accounts({
            payer: payer,
            config: { config: this.pdas.configAccount() },
            peer: this.pdas.transceiverPeerAccount(emitterChain),
            vaa: sdk_solana_core_1.utils.derivePostedVaaKey(this.wormholeId, Buffer.from(wormholeNTT.hash)),
            transceiverMessage: this.pdas.transceiverMessageAccount(emitterChain, nttMessage.id),
        })
            .instruction();
    }
    async createRedeemInstruction(payer, wormholeNTT) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const nttMessage = wormholeNTT.payload.nttManagerPayload;
        const emitterChain = wormholeNTT.emitterChain;
        const nttManagerPeer = this.pdas.peerAccount(emitterChain);
        const inboxRateLimit = this.pdas.inboxRateLimitAccount(emitterChain);
        const inboxItem = this.pdas.inboxItemAccount(emitterChain, nttMessage);
        return await this.program.methods
            .redeem({})
            .accounts({
            payer: payer,
            config: this.pdas.configAccount(),
            peer: nttManagerPeer,
            transceiverMessage: this.pdas.transceiverMessageAccount(emitterChain, nttMessage.id),
            transceiver: this.pdas.registeredTransceiver(this.program.programId),
            mint: config.mint,
            inboxItem,
            inboxRateLimit,
            outboxRateLimit: this.pdas.outboxRateLimitAccount(),
        })
            .instruction();
    }
    async createReleaseInboundMintInstruction(args) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const inboxItem = this.pdas.inboxItemAccount(args.chain, args.nttMessage);
        const recipientAddress = args.recipient ??
            (await this.getInboundQueuedTransfer(args.chain, args.nttMessage)).recipient
                .toNative(this.chain)
                .unwrap();
        return await this.program.methods
            .releaseInboundMint({
            revertOnDelay: args.revertOnDelay,
        })
            .accounts({
            common: {
                payer: args.payer,
                config: { config: this.pdas.configAccount() },
                inboxItem,
                recipient: (0, spl_token_1.getAssociatedTokenAddressSync)(config.mint, recipientAddress),
                mint: config.mint,
                tokenAuthority: this.pdas.tokenAuthority(),
            },
        })
            .instruction();
    }
    async createReleaseInboundUnlockInstruction(args) {
        const config = await this.getConfig();
        if (config.paused)
            throw new Error('Contract is paused');
        const recipientAddress = args.recipient ??
            (await this.getInboundQueuedTransfer(args.chain, args.nttMessage)).recipient
                .toNative(this.chain)
                .unwrap();
        const inboxItem = this.pdas.inboxItemAccount(args.chain, args.nttMessage);
        console.log('Inbox Item', inboxItem.toBase58());
        return await this.program.methods
            .releaseInboundUnlock({
            revertOnDelay: args.revertOnDelay,
        })
            .accounts({
            common: {
                payer: args.payer,
                config: { config: this.pdas.configAccount() },
                inboxItem: inboxItem,
                recipient: (0, spl_token_1.getAssociatedTokenAddressSync)(config.mint, recipientAddress),
                mint: config.mint,
                tokenAuthority: this.pdas.tokenAuthority(),
            },
            custody: config.custody,
        })
            .instruction();
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new sdk_solana_1.SolanaUnsignedTransaction(txReq, this.network, this.chain, description, parallelizable);
    }
}
exports.SolanaNtt = SolanaNtt;
//# sourceMappingURL=ntt.js.map