"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nttAddresses = exports.derivePda = void 0;
const web3_js_1 = require("@solana/web3.js");
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
function derivePda(seeds, programId) {
    const toBytes = (s) => typeof s === 'string' ? sdk_connect_1.encoding.bytes.encode(s) : s;
    return web3_js_1.PublicKey.findProgramAddressSync(Array.isArray(seeds) ? seeds.map(toBytes) : [toBytes(seeds)], new web3_js_1.PublicKey(programId))[0];
}
exports.derivePda = derivePda;
const chainToBytes = (chain) => sdk_connect_1.encoding.bignum.toBytes((0, sdk_connect_1.toChainId)(chain), 2);
// TODO: memoize?
const nttAddresses = (programId) => {
    const configAccount = () => derivePda('config', programId);
    const emitterAccount = () => derivePda('emitter', programId);
    const inboxRateLimitAccount = (chain) => derivePda(['inbox_rate_limit', chainToBytes(chain)], programId);
    const inboxItemAccount = (chain, nttMessage) => derivePda(['inbox_item', sdk_connect_1.Ntt.messageDigest(chain, nttMessage)], programId);
    const outboxRateLimitAccount = () => derivePda('outbox_rate_limit', programId);
    const tokenAuthority = () => derivePda('token_authority', programId);
    const peerAccount = (chain) => derivePda(['peer', chainToBytes(chain)], programId);
    const transceiverPeerAccount = (chain) => derivePda(['transceiver_peer', chainToBytes(chain)], programId);
    const registeredTransceiver = (transceiver) => derivePda(['registered_transceiver', transceiver.toBytes()], programId);
    const transceiverMessageAccount = (chain, id) => derivePda(['transceiver_message', chainToBytes(chain), id], programId);
    const wormholeMessageAccount = (outboxItem) => derivePda(['message', outboxItem.toBytes()], programId);
    const sessionAuthority = (sender, args) => derivePda([
        'session_authority',
        sender.toBytes(),
        (0, sdk_connect_1.keccak256)(sdk_connect_1.encoding.bytes.concat(new Uint8Array(args.amount.toArrayLike(Buffer, 'be', 8)), chainToBytes(args.recipientChain.id), new Uint8Array(args.recipientAddress), new Uint8Array([args.shouldQueue ? 1 : 0]))),
    ], programId);
    return {
        configAccount,
        outboxRateLimitAccount,
        inboxRateLimitAccount,
        inboxItemAccount,
        sessionAuthority,
        tokenAuthority,
        emitterAccount,
        wormholeMessageAccount,
        peerAccount,
        transceiverPeerAccount,
        transceiverMessageAccount,
        registeredTransceiver,
    };
};
exports.nttAddresses = nttAddresses;
//# sourceMappingURL=utils.js.map