import { IdlAccounts, Program } from '@coral-xyz/anchor';
import { Connection, PublicKey, TransactionInstruction } from '@solana/web3.js';
import { AccountAddress, Chain, ChainAddress, Network, Ntt, NttTransceiver, ProtocolInitializer, TokenAddress, UnsignedTransaction, WormholeNttTransceiver } from '@wormhole-foundation/sdk-connect';
import { SolanaChains, SolanaTransaction, SolanaUnsignedTransaction } from '@wormhole-foundation/sdk-solana';
import { SolanaWormholeCore } from '@wormhole-foundation/sdk-solana-core';
import type { NativeTokenTransfer } from './anchor-idl/index.js';
import { TransferArgs, nttAddresses } from './utils.js';
interface NttContracts {
    manager: string;
    transceiver: {
        wormhole?: string;
    };
}
export type Config = IdlAccounts<NativeTokenTransfer>['config'];
export type InboxItem = IdlAccounts<NativeTokenTransfer>['inboxItem'];
export declare function solanaNttProtocolFactory(token: string): ProtocolInitializer<'Solana', 'Ntt'>;
export declare class SolanaNttWormholeTransceiver<N extends Network, C extends SolanaChains> implements NttTransceiver<N, C, WormholeNttTransceiver.VAA> {
    readonly manager: SolanaNtt<N, C>;
    readonly address: string;
    constructor(manager: SolanaNtt<N, C>, address: string);
    receive(attestation: WormholeNttTransceiver.VAA, sender?: AccountAddress<C> | undefined): AsyncGenerator<UnsignedTransaction<N, C>, any, unknown>;
}
export declare class SolanaNtt<N extends Network, C extends SolanaChains> implements Ntt<N, C> {
    readonly network: N;
    readonly chain: C;
    readonly connection: Connection;
    readonly wormholeId: string;
    readonly contracts: NttContracts;
    core: SolanaWormholeCore<N, C>;
    xcvrs: SolanaNttWormholeTransceiver<N, C>[];
    program: Program<NativeTokenTransfer>;
    pdas: ReturnType<typeof nttAddresses>;
    config?: Config;
    constructor(network: N, chain: C, connection: Connection, wormholeId: string, contracts: NttContracts);
    getConfig(): Promise<Config>;
    transfer(sender: AccountAddress<C>, amount: bigint, destination: ChainAddress, queue: boolean): AsyncGenerator<UnsignedTransaction<N, C>, any, unknown>;
    redeem(attestations: Ntt.Attestation[], payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    getCurrentOutboundCapacity(): Promise<bigint>;
    getCurrentInboundCapacity(fromChain: Chain): Promise<bigint>;
    completeInboundQueuedTransfer(fromChain: Chain, transceiverMessage: Ntt.Message, token: TokenAddress<C>, payer: AccountAddress<C>): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    getInboundQueuedTransfer(chain: Chain, nttMessage: Ntt.Message): Promise<Ntt.InboundQueuedTransfer<C> | null>;
    createTransferLockInstruction(args: {
        transferArgs: TransferArgs;
        payer: PublicKey;
        from: PublicKey;
        fromAuthority: PublicKey;
        outboxItem: PublicKey;
        config?: Config;
    }): Promise<TransactionInstruction>;
    createTransferBurnInstruction(args: {
        transferArgs: TransferArgs;
        payer: PublicKey;
        from: PublicKey;
        fromAuthority: PublicKey;
        outboxItem: PublicKey;
        config?: Config;
    }): Promise<TransactionInstruction>;
    createReleaseOutboundInstruction(args: {
        payer: PublicKey;
        outboxItem: PublicKey;
        revertOnDelay: boolean;
    }): Promise<TransactionInstruction>;
    createReceiveWormholeMessageInstruction(payer: PublicKey, wormholeNTT: WormholeNttTransceiver.VAA): Promise<TransactionInstruction>;
    createRedeemInstruction(payer: PublicKey, wormholeNTT: WormholeNttTransceiver.VAA): Promise<TransactionInstruction>;
    createReleaseInboundMintInstruction(args: {
        payer: PublicKey;
        chain: Chain;
        nttMessage: Ntt.Message;
        revertOnDelay: boolean;
        recipient?: PublicKey;
    }): Promise<TransactionInstruction>;
    createReleaseInboundUnlockInstruction(args: {
        payer: PublicKey;
        chain: Chain;
        nttMessage: Ntt.Message;
        revertOnDelay: boolean;
        recipient?: PublicKey;
    }): Promise<TransactionInstruction>;
    createUnsignedTx(txReq: SolanaTransaction, description: string, parallelizable?: boolean): SolanaUnsignedTransaction<N, C>;
}
export {};
//# sourceMappingURL=ntt.d.ts.map