import { PublicKey } from '@solana/web3.js';
import { Ntt, encoding, keccak256, toChainId, } from '@wormhole-foundation/sdk-connect';
export function derivePda(seeds, programId) {
    const toBytes = (s) => typeof s === 'string' ? encoding.bytes.encode(s) : s;
    return PublicKey.findProgramAddressSync(Array.isArray(seeds) ? seeds.map(toBytes) : [toBytes(seeds)], new PublicKey(programId))[0];
}
const chainToBytes = (chain) => encoding.bignum.toBytes(toChainId(chain), 2);
// TODO: memoize?
export const nttAddresses = (programId) => {
    const configAccount = () => derivePda('config', programId);
    const emitterAccount = () => derivePda('emitter', programId);
    const inboxRateLimitAccount = (chain) => derivePda(['inbox_rate_limit', chainToBytes(chain)], programId);
    const inboxItemAccount = (chain, nttMessage) => derivePda(['inbox_item', Ntt.messageDigest(chain, nttMessage)], programId);
    const outboxRateLimitAccount = () => derivePda('outbox_rate_limit', programId);
    const tokenAuthority = () => derivePda('token_authority', programId);
    const peerAccount = (chain) => derivePda(['peer', chainToBytes(chain)], programId);
    const transceiverPeerAccount = (chain) => derivePda(['transceiver_peer', chainToBytes(chain)], programId);
    const registeredTransceiver = (transceiver) => derivePda(['registered_transceiver', transceiver.toBytes()], programId);
    const transceiverMessageAccount = (chain, id) => derivePda(['transceiver_message', chainToBytes(chain), id], programId);
    const wormholeMessageAccount = (outboxItem) => derivePda(['message', outboxItem.toBytes()], programId);
    const sessionAuthority = (sender, args) => derivePda([
        'session_authority',
        sender.toBytes(),
        keccak256(encoding.bytes.concat(new Uint8Array(args.amount.toArrayLike(Buffer, 'be', 8)), chainToBytes(args.recipientChain.id), new Uint8Array(args.recipientAddress), new Uint8Array([args.shouldQueue ? 1 : 0]))),
    ], programId);
    return {
        configAccount,
        outboxRateLimitAccount,
        inboxRateLimitAccount,
        inboxItemAccount,
        sessionAuthority,
        tokenAuthority,
        emitterAccount,
        wormholeMessageAccount,
        peerAccount,
        transceiverPeerAccount,
        transceiverMessageAccount,
        registeredTransceiver,
    };
};
//# sourceMappingURL=utils.js.map